<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 10/13/2004 *
 * ===================                          Last change: 10/13/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : transfer_functions.php                           *
 * -------------------------------------------------------------------- *
 * Short description : Special functions for transfer extension         *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Spezielle Funktion fuer transfer-Erweiterung     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Purge expired transfer entries
function autoPurgeTransfers ($max, $age) {
	// First get total in-going lines
	$result = sqlQuery('SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_transfers_in` ORDER BY `id` ASC', __FUNCTION__, __LINE__);
	if (sqlNumRows($result) > $max) {
		// Update overdue transfers
		$remove = sqlNumRows($result) - $max;

		// This will make it really old, so the final removal query will find it
		$result = sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_user_transfers_in` SET `time_trans`=0 ORDER BY `id` ASC LIMIT %s",
			array($remove), __FUNCTION__, __LINE__);
	} // END - if

	// Second get total out-going lines
	$result = sqlQuery('SELECT `id` FROM `{?_MYSQL_PREFIX?}_user_transfers_out` ORDER BY `id` ASC', __FUNCTION__, __LINE__);
	if (sqlNumRows($result) > $max) {
		// Update overdue transfers
		$remove = sqlNumRows($result) - $max;

		// This will make it really old, so the final removal query will find it
		$result = sqlQueryEscaped("UPDATE `{?_MYSQL_PREFIX?}_user_transfers_out` SET `time_trans`=0 ORDER BY `id` ASC LIMIT %s",
			array($remove), __FUNCTION__, __LINE__);
	} // END - if

	// Remove old in-going transfers
	$result = sqlQueryEscaped("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_user_transfers_in` WHERE `time_trans` < (UNIX_TIMESTAMP() - %s)",
		array($age), __FUNCTION__, __LINE__);
	$removed = sqlAffectedRows();

	// Remove old out-going transfers
	$result = sqlQueryEscaped("DELETE LOW_PRIORITY FROM `{?_MYSQL_PREFIX?}_user_transfers_out` WHERE `time_trans` < (UNIX_TIMESTAMP() - %s)",
		array($age), __FUNCTION__, __LINE__);
	$removed += sqlAffectedRows();

	// Only send email to admin(s) when we have removed entries
	if ($removed > 0) {
		sendAdminNotification('{--ADMIN_TRANSFER_AUTOPURGE_SUBJECT--}', 'admin_transfer_ap', $removed);
	} // END - if
}

//-----------------------------------------------------------------------------
//                              Wrapper functions
//-----------------------------------------------------------------------------

// Wrapper function for transfer_code
function getTransferCode () {
	// Is cache set?
	if (!isset($GLOBALS[__FUNCTION__])) {
		// Get config entry
		$GLOBALS[__FUNCTION__] = getConfig('transfer_code');
	} // END - if

	// Return cache
	return $GLOBALS[__FUNCTION__];
}

// [EOF]
?>
