<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 02/12/2004 *
 * ===================                          Last change: 11/12/2004 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : what-theme_import.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Import of new themes                             *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Importieren von neuen Themes                     *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009 - 2015 by Mailer Developer Team                   *
 * For more information visit: http://mxchange.org                      *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if ((!defined('__SECURITY')) || (!isAdmin())) {
	exit();
} // END - if

// Add description as navigation point
addYouAreHereLink('admin', __FILE__);

// Switch to testing mode
$GLOBALS['theme_mode'] = 'test';

// Import selected theme if not present
if (isPostRequestElementSet('theme')) {
	// Check if theme is there
	if (!ifThemeExists(postRequestElement('theme'))) {
		// Is the theme readable?
		if (isThemeReadable(postRequestElement('theme'))) {
			// Load the theme header file
			loadInclude(sprintf('theme/%s/theme.php', sqlEscapeString(postRequestElement('theme'))));

			// Register it ith the exchange
			sqlQueryEscaped("INSERT INTO `{?_MYSQL_PREFIX?}_themes` (`theme_path`, `theme_active`, `theme_ver`, `theme_name`)
VALUES ('%s','N','%s','%s')",
				array(
					postRequestElement('theme'),
					$GLOBALS['theme_data']['version'],
					$GLOBALS['theme_data']['name']
				), __FILE__, __LINE__);

			// Destroy cache
			rebuildCache('themes', 'themes');

			// Prepare message
			$message = '{%message,ADMIN_THEME_IMPORTED=' . postRequestElement('theme') . '%}';
		} else {
			// Include file not found
			$message = '{%message,ADMIN_THEME_INC_404=' . postRequestElement('theme') . '%}';
		}
	} else {
		// Theme already imported
		$message = '{%message,ADMIN_THEME_ALREADY_INSTALLED=' . postRequestElement('theme') . '%}';
	}

	// Output message
	displayMessage($message);
} // END - if

// Initialize array
$themes = array(
	'theme_unix'    => array(), // Unix name from filesystem
	'theme_name'    => array(), // Title
	'theme_author'  => array(), // Theme author's name
	'theme_email'   => array(), // Author's email address
	'theme_url'     => array(), // URL were you can download it from
	'theme_version' => array(), // Version number of theme
);

// Read directory "themes"
$includes = getArrayFromDirectory('theme/', '', FALSE, TRUE, array('css', 'images'));

// Walk through all entries and add it
foreach ($includes as $inc) {
	// Get directory from it
	$dir = basename(dirname($inc));

	// Load include file
	loadInclude($inc);

	// Add found theme to array
	array_push($themes['theme_unix']   , $dir);
	array_push($themes['theme_name']   , $GLOBALS['theme_data']['name']);
	array_push($themes['theme_author'] , $GLOBALS['theme_data']['author']);
	array_push($themes['theme_email']  , $GLOBALS['theme_data']['email']);
	array_push($themes['theme_url']    , $GLOBALS['theme_data']['url']);
	array_push($themes['theme_version'], $GLOBALS['theme_data']['version']);
} // END - while

// Sort array by Uni* name
array_pk_sort($themes, array('theme_name'));

// Generate output lines for the template
$OUT = '';
foreach ($themes['theme_unix'] as $key => $unix) {
	// Already installed is default
	$formContent = '<div class="notice">{%message,ADMIN_THEME_ALREADY_INSTALLED=' . $unix . '%}</div>';

	// Check if current theme is already imported or not
	if (!ifThemeExists($unix)) {
		// Theme not installed
		$formContent = loadTemplate('admin_import_theme_form', TRUE, $unix);
	} // END - if

	// Prepare content
	$content = array(
		'unix'          => $unix,
		'theme_name'    => $themes['theme_name'][$key],
		'theme_email'   => $themes['theme_email'][$key],
		'theme_author'  => $themes['theme_author'][$key],
		'theme_url'     => $themes['theme_url'][$key],
		'theme_version' => $themes['theme_version'][$key],
		'form_content'  => $formContent
	);

	// Add row template
	$OUT .= loadTemplate('admin_import_theme_row', TRUE, $content);
} // END - foreach

if (empty($OUT)) {
	// No themes found???
	$OUT .= loadTemplate('admin_import_theme_none', TRUE, displayMessage('{--ADMIN_NO_THEMES_FOUND--}', TRUE));
} // END - if

// Load template
loadTemplate('admin_import_theme', FALSE, $OUT);

// [EOF]
?>
